"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const dc_messaging_1 = require("dc-messaging");
const PingService_1 = require("../dapps/PingService");
const mocha_1 = require("mocha");
const chai_1 = require("chai");
const dc_logging_1 = require("dc-logging");
const events_1 = require("events");
const randomString = () => Math.random()
    .toString(36)
    .substring(2, 15) +
    Math.random()
        .toString(36)
        .substring(2, 15);
function sleep(ms) {
    return new Promise(resolve => {
        setTimeout(resolve, ms);
    });
}
const log = new dc_logging_1.Logger("PingService test");
const apiRoomAddress = [randomString(), randomString()];
class RemoteClient extends events_1.EventEmitter {
    start(pingService) {
        const acceptPing = (event, data) => {
            log.debug({ event, data });
            chai_1.expect(data.apiRoomAddress).to.be.a("string");
            /* tslint:disable-next-line  */
            chai_1.expect(apiRoomAddress.includes(data.apiRoomAddress)).to.be.true;
        };
        this.pingService = pingService;
        this.proxyOn(PingService_1.PingService.EVENT_JOIN, data => {
            acceptPing(PingService_1.PingService.EVENT_JOIN, data);
        });
        this.proxyOn(PingService_1.PingService.EVENT_EXIT, data => acceptPing(PingService_1.PingService.EVENT_EXIT, data));
        this.proxyOn(PingService_1.PingService.EVENT_PONG, data => acceptPing(PingService_1.PingService.EVENT_PONG, data));
        pingService.emit(PingService_1.PingService.EVENT_PING, null);
        return this;
    }
    proxyOn(event, func) {
        this.pingService.on(event, func);
    }
}
const test = (transportProviderFactory) => mocha_1.describe(transportProviderFactory.toString(), () => {
    const pingService = [];
    const pingProvider = [];
    const timeout = 400;
    let clientService;
    let clientProvider;
    const platformId = randomString();
    mocha_1.it(`Start ${apiRoomAddress.length} node with PingService`, async () => {
        for (const address of apiRoomAddress) {
            const provider = await transportProviderFactory.create();
            const params = {
                platformId,
                apiRoomAddress: address,
                blockchainNetwork: randomString(),
                ethAddress: randomString()
            };
            const service = new PingService_1.PingService().start(provider, params);
            /* tslint:disable-next-line  */
            chai_1.expect(service.isStarted()).to.be.true;
            pingService.push(service);
            pingProvider.push(provider);
        }
    });
    mocha_1.it(`Start node with ClientService`, async () => {
        const provider = await transportProviderFactory.create();
        const peer = await provider.getRemoteInterface(platformId);
        const service = await new RemoteClient().start(peer);
        clientService = service;
        clientProvider = provider;
        log.debug('Client started.');
        await sleep(2000);
    });
    mocha_1.it("Stop PingService", async () => {
        for (const service of pingService) {
            await service.stop();
            /* tslint:disable-next-line */
            chai_1.expect(service.isStarted()).to.be.false;
        }
        await sleep(1000);
        for (const provider of pingProvider) {
            await provider.destroy();
        }
    });
    mocha_1.it('Client leave room', async () => {
        await clientProvider.destroy();
    });
});
mocha_1.describe('PingService test', () => {
    mocha_1.it('IPFS', () => {
        const factory = new dc_messaging_1.TransportProviderFactory(dc_messaging_1.TransportType.IPFS);
        test(factory);
    });
    mocha_1.it('WS', () => {
        const factory = new dc_messaging_1.TransportProviderFactory(dc_messaging_1.TransportType.WS);
        test(factory);
    });
    // factory.setType(TransportType.WS)
    // test(factory)
});
//# sourceMappingURL=PingService.test.js.map