"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Bankroller_1 = __importDefault(require("../dapps/Bankroller"));
const mocha_1 = require("mocha");
const chai_1 = require("chai");
const dc_configs_1 = require("dc-configs");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const dc_messaging_1 = require("dc-messaging");
const dc_logging_1 = require("dc-logging");
const log = new dc_logging_1.Logger("Bankroller test");
// TODO: create expect checks
const randomString = () => Math.random()
    .toString(36)
    .substring(2, 15) +
    Math.random()
        .toString(36)
        .substring(2, 15);
const EXAMPLE_GAME_PATH = "FTE1";
const EXAMPLE_GAME_NAME = "DCGame_FTE_v1";
const createFakeGame = (name) => {
    const DAppsPath = dc_configs_1.config.default.DAppsPath;
    const openDir = path_1.default.join(DAppsPath, EXAMPLE_GAME_PATH);
    const files = fs_1.default.readdirSync(openDir).map(fileName => {
        const filePath = path_1.default.join(openDir, fileName);
        const { size } = fs_1.default.statSync(filePath);
        return { fileName, fileData: fs_1.default.readFileSync(filePath).toString('base64'), fileSize: size };
    });
    return { name, files };
};
const checkSize = (game) => {
    const DAppsPath = dc_configs_1.config.default.DAppsPath;
    const uploadedDir = path_1.default.join(DAppsPath, game.name); // directory game uploaded
    /* tslint:disable-next-line */
    chai_1.expect(fs_1.default.existsSync(uploadedDir)).to.be.true;
    fs_1.default.readdirSync(uploadedDir).map(fileName => {
        const filePath = path_1.default.join(uploadedDir, fileName);
        const { size } = fs_1.default.statSync(filePath);
        const { fileSize } = game.files.find(file => file.fileName === fileName);
        chai_1.expect(size).to.be.equal(fileSize);
    });
};
const bankrollerTest = (type) => mocha_1.describe(`Transport layer ${dc_messaging_1.TransportType[type]}`, () => {
    let provider;
    let bankroller;
    let game;
    mocha_1.it(`Start bankroller`, async () => {
        const factory = new dc_messaging_1.TransportProviderFactory(type);
        provider = await factory.create();
        bankroller = await new Bankroller_1.default().start(provider);
        /* tslint:disable-next-line */
        chai_1.expect(bankroller.isStarted()).to.be.true;
    });
    mocha_1.it("Upload game", async () => {
        game = createFakeGame(randomString());
        const result = await bankroller.uploadGame(game);
        chai_1.expect(result.status).to.equal(Bankroller_1.default.STATUS_SUCCESS);
        checkSize(game);
    });
    mocha_1.it('Get games list', () => {
        const list = bankroller.getGames();
        console.log(list);
        /* tslint:disable-next-line */
        chai_1.expect(list.length !== 0).to.be.true;
        /* tslint:disable-next-line */
        chai_1.expect(list.findIndex(item => item.name === EXAMPLE_GAME_NAME) !== -1).to.be.true;
    });
    mocha_1.it("Get game instances", () => {
        const instances = bankroller.getGameInstances(EXAMPLE_GAME_NAME);
        chai_1.expect(instances).to.be.a('array');
    });
    mocha_1.it("Success reload game", async () => {
        const reloadGame = Object.assign({}, game, { reload: true });
        const result = await bankroller.uploadGame(reloadGame);
        chai_1.expect(result.status).to.equal(Bankroller_1.default.STATUS_SUCCESS);
        checkSize(reloadGame);
    });
    mocha_1.it("Error reload game", async () => {
        let error;
        try {
            await bankroller.uploadGame(game);
        }
        catch (e) {
            error = e;
        }
        chai_1.expect(error).to.be.an.instanceof(Error);
    });
    mocha_1.it("Unload game", async () => {
        const result = await bankroller.unloadGame(game.name);
        chai_1.expect(result.status).to.equal(Bankroller_1.default.STATUS_SUCCESS);
        const DAppsPath = dc_configs_1.config.default.DAppsPath;
        const uploadedDir = path_1.default.join(DAppsPath, game.name);
        /* tslint:disable-next-line */
        chai_1.expect(fs_1.default.existsSync(uploadedDir)).to.be.false;
    });
    mocha_1.it("Stop bankroller", async () => {
        await bankroller.stop();
        /* tslint:disable-next-line */
        chai_1.expect(bankroller.isStarted()).to.be.false;
        await provider.destroy();
    });
});
const bankrollerRemoteTest = (type) => {
    let provider;
    let remoteProvider;
    let bankroller;
    let remoteBankroller;
    let game;
    mocha_1.it(`Start bankroller`, async () => {
        const factory = new dc_messaging_1.TransportProviderFactory(type);
        provider = await factory.create();
        bankroller = await new Bankroller_1.default().start(provider);
        /* tslint:disable-next-line */
        chai_1.expect(bankroller.isStarted()).to.be.true;
    });
    mocha_1.it(`Get remote interface`, async () => {
        const factory = new dc_messaging_1.TransportProviderFactory(type);
        remoteProvider = await factory.create();
        remoteBankroller = await remoteProvider.getRemoteInterface(bankroller.getApiRoomAddress());
        /* tslint:disable-next-line */
        chai_1.expect(await remoteBankroller.isStarted()).to.be.true;
    });
    mocha_1.it('Remote upload game', async () => {
        game = createFakeGame(randomString());
        const result = await remoteBankroller.uploadGame(game);
        chai_1.expect(result.status).to.equal(Bankroller_1.default.STATUS_SUCCESS);
        checkSize(game);
    });
    mocha_1.it("Unload game", async () => {
        const result = await remoteBankroller.unloadGame(game.name);
        chai_1.expect(result.status).to.equal(Bankroller_1.default.STATUS_SUCCESS);
        const DAppsPath = dc_configs_1.config.default.DAppsPath;
        const uploadedDir = path_1.default.join(DAppsPath, game.name);
        /* tslint:disable-next-line */
        chai_1.expect(fs_1.default.existsSync(uploadedDir)).to.be.false;
    });
    mocha_1.it(`Stop remote interface`, async () => {
        await remoteProvider.destroy();
    });
    mocha_1.it("Stop bankroller", async () => {
        await bankroller.stop();
        /* tslint:disable-next-line */
        chai_1.expect(bankroller.isStarted()).to.be.false;
        await provider.destroy();
    });
};
bankrollerTest(dc_messaging_1.TransportType.IPFS);
bankrollerTest(dc_messaging_1.TransportType.WS);
bankrollerTest(dc_messaging_1.TransportType.DIRECT);
// bankrollerRemoteTest(TransportType.IPFS)
// bankrollerRemoteTest(TransportType.WS)
// bankrollerRemoteTest(TransportType.DIRECT)
//# sourceMappingURL=bankroller.test.js.map