"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const rimraf_1 = __importDefault(require("rimraf"));
const MANIFEST_FILENAME = "dapp.manifest";
exports.checkFileExists = (fileName, maybeExtension) => {
    for (let i = 0; i < maybeExtension.length; i++) {
        const pathToFile = `${fileName}${maybeExtension[i]}`;
        if (fs_1.default.existsSync(pathToFile)) {
            return pathToFile;
        }
    }
    return null;
};
exports.getSubDirectories = (directoryPath) => {
    return fs_1.default
        .readdirSync(directoryPath)
        .map(subDir => path_1.default.join(directoryPath, subDir))
        .filter(subDir => fs_1.default.lstatSync(subDir).isDirectory() && subDir);
};
exports.removeDir = (directoryPath) => {
    rimraf_1.default.sync(directoryPath);
};
exports.saveFilesToNewDir = (directoryPath, files) => {
    let result = true;
    fs_1.default.mkdirSync(directoryPath);
    try {
        files.forEach(file => {
            fs_1.default.writeFileSync(path_1.default.join(directoryPath, file.fileName), Buffer.from(file.fileData, 'base64'), 'utf-8');
        });
    }
    catch (error) {
        fs_1.default.rmdirSync(directoryPath);
        result = false;
    }
    return result;
};
exports.loadLogic = async (directoryPath) => {
    const manifestPath = `${directoryPath}/${MANIFEST_FILENAME}`;
    const manifestFoundPath = exports.checkFileExists(manifestPath, [".js", "", ".json"]);
    if (!manifestFoundPath) {
        throw new Error(`Manifest file not found ${manifestPath}`);
    }
    const manifest = manifestFoundPath.endsWith(".js")
        ? require(manifestFoundPath)
        : JSON.parse(fs_1.default.readFileSync(manifestFoundPath).toString());
    if (typeof manifest !== "object" ||
        manifest.disable ||
        manifest.disabled ||
        manifest.enable === false) {
        return { manifest, gameLogicFunction: null };
    }
    const logicPath = path_1.default.join(directoryPath, manifest.logic);
    const logicFoundPath = exports.checkFileExists(logicPath, [
        ".js",
        ".mjs",
        "",
        ".json"
    ]);
    if (!logicFoundPath) {
        throw new Error(`Manifest file not found ${logicFoundPath}`);
    }
    const gameLogicFunction = require(logicFoundPath);
    if (!gameLogicFunction) {
        throw new Error(`Error loading logic from directory ${directoryPath}`);
    }
    return { manifest: Object.assign({}, manifest), gameLogicFunction };
};
//# sourceMappingURL=FileUtils.js.map