"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
const dc_logging_1 = require("dc-logging");
const log = new dc_logging_1.Logger("PingService");
class PingService extends events_1.EventEmitter {
    start(transportProvider, params) {
        if (this._started) {
            throw new Error("PingService allready started");
        }
        this._transportProvider = transportProvider;
        this._params = params;
        this._address = params.platformId;
        transportProvider.exposeSevice(this._address, this, true);
        const pingResponce = this.ping();
        this.on(PingService.EVENT_PING, () => {
            log.debug(`Ping request, emit PONG - ${this._params.apiRoomAddress}`);
            this.emit(PingService.EVENT_PONG, pingResponce);
        });
        this.on("connected", ({ id, address }) => {
            log.debug(`Peer connected, emit remote JOIN - ${this._params.apiRoomAddress}`);
            transportProvider.emitRemote(address, id, PingService.EVENT_JOIN, pingResponce);
        });
        this._started = true;
        log.debug(`Ping service started.`);
        return this;
    }
    eventNames() {
        return [
            PingService.EVENT_PING,
            PingService.EVENT_PONG,
            PingService.EVENT_JOIN,
            PingService.EVENT_EXIT
        ];
    }
    ping() {
        return this._params;
    }
    isStarted() {
        return this._started;
    }
    async stop() {
        log.debug(`Ping service stop, emit EXIT - ${this._params.apiRoomAddress}`);
        this.emit(PingService.EVENT_EXIT, this.ping());
        await this._transportProvider.stopService(this._address);
        this._started = false;
    }
}
PingService.EVENT_PING = "bankrollerPing";
PingService.EVENT_PONG = "bankrollerPong";
PingService.EVENT_JOIN = "bankrollerJoin";
PingService.EVENT_EXIT = "bankrollerExit";
exports.PingService = PingService;
//# sourceMappingURL=PingService.js.map